package ch.hepia.api.transport;

import org.json.JSONObject;

import java.io.Serializable;

public final class Coordinates implements Serializable {
    private Type type;
    private double x;
    private double y;

    /**
     * Constructor of the class Coordinates
     * A coordinates is represented by a type of place and a point (x, y)
     * @param type type of place we are a these coordinates
     * @param x coordinate in x
     * @param y coordinate in y
     */
    private Coordinates(Type type, double x, double y){
        this.type = type;
        this.x = x;
        this.y = y;
    }

    @Override
    public String toString(){
        StringBuilder string = new StringBuilder();
        return string.append(type.toString()).append("au point (").append(x).append(", ").append(y).append(")").toString();
    }

    /**
     * Get the x coordinate of the place (x, y)
     * @return x
     */
    public double getX(){
        return x;
    }

    /**
     * Get the y coordinate of the place (x, y)
     * @return y
     */
    public double getY(){
        return y;
    }

    /**
     * Builder of Coordinates object
     */
    public static class CoordinatesBuilder{
        private JSONObject datas;

        /**
         * Constructor of the class CoordinatesBuilder
         * Stock Datas from a JSON object given
         * @param datas contains datas to construct a Coordinates object
         */
        public CoordinatesBuilder(JSONObject datas){
            this.datas = datas;
        }

        /**
         * Build a Coordinates object from the datas obtained
         * @return a new Coordinates object
         */
        public Coordinates build(){
            Type type = Type.valueOf(datas.getString("type"));
            double x = datas.getDouble("x");
            double y = datas.getDouble("y");

            return new Coordinates(type, x, y);
        }
    }
}
