package ch.hepia.api.transport;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.Serializable;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

public final class Journey implements Serializable {
    private String name;
    private String category;
    private int categoryCode;
    private int number;
    private String operator;
    private String to;
    private List<Stop> passList;
    private int capacity1st;
    private int capacity2nd;

    /**
     * Constructor of the Journey Object
     * @param name
     * @param category what transport category the journey is
     * @param categoryCode
     * @param number
     * @param operator which company operates the journey
     * @param to destination of the journey
     * @param passList Where the journey will pass through
     * @param capacity1st number of seats inst class
     * @param capacity2nd number of seats in nd class
     */
    private Journey(String name, String category, int categoryCode, int number, String operator, String to, List<Stop> passList, int capacity1st, int capacity2nd){
        this.name = name;
        this.category = category;
        this.categoryCode = categoryCode;
        this.number = number;
        this.operator = operator;
        this.to = to;
        this.passList = new ArrayList<>(passList);
        this.capacity1st = capacity1st;
        this.capacity2nd = capacity2nd;
    }

    /**
     * Get the name of the journey
     * @return name
     */
    public String getName(){
        return name;
    }

    /**
     * Get the Category of the transport of the Journey
     * @return category
     */
    public String getCategory(){
        return category;
    }

    /**
     * Get the categoryCode affiliated with the Journey
     * @return categoryCode
     */
    public int getCategoryCode(){
        return categoryCode;
    }

    /**
     * Get where the Journey is headed to
     * @return to
     */
    public String getTo(){
        return to;
    }

    /**
     * Get all the Stops where the journey will pass through
     * @return passList
     */
    public List<Stop> getPassList(){
        return new ArrayList<>(passList);
    }

    /**
     * Builder of Journey object
     */
    public static class JourneyBuilder{
        private JSONObject datas;

        /**
         * Constructor of the class JourneyBuilder
         * Stock Datas from a JSON object given
         * @param datas contains datas to construct a Journey object
         */
        public JourneyBuilder(JSONObject datas){
            this.datas = datas;
        }

        /**
         * Build a new Journey object from the datas obtained
         * @return a new Journey Object
         * @throws ParseException
         */
        public Journey build() throws ParseException {
            String name = datas.getString("name");
            String category = datas.getString("category");
            int categoryCode = datas.getInt("categoryCode");
            int number = datas.getInt("number");
            String operator = datas.getString("operator");

            String to = datas.getString("to");

            JSONArray passListJSON = datas.getJSONArray("passList");
            List<Stop> passList = new ArrayList<>();
            passListJSON.forEach( o -> {
                JSONObject k = (JSONObject) o;
                Stop.StopBuilder stop = new Stop.StopBuilder(k);
                try {
                    passList.add(stop.build());
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            });

            int capacity1st = datas.getInt("capacity1st");
            int capacity2nd = datas.getInt("capacity2nd");

            return new Journey(name, category, categoryCode, number, operator, to, passList, capacity1st, capacity2nd);
        }
    }
}
