package ch.hepia.api.transport;

import org.json.JSONObject;

import java.io.Serializable;

public final class Location implements Serializable {
    private int id;
    private Type type;
    private String name;
    private double score;
    private Coordinates where;
    private double distance;

    /**
     * Constructor of the class Location
     * @param id unique to the Location
     * @param type type of place the Location is
     * @param name
     * @param score accuracy of the result (may not be used)
     * @param where Localisation of the Location
     * @param distance distance to the Location if there has been Coordinates (may not be used)
     */
    private Location(int id, Type type, String name, double score, Coordinates where, double distance){
        this.id = id;
        this.type = type;
        this.name = name;
        this.score = score;
        this.where = where;
        this.distance = distance;
    }

    /**
     * Get the coordinates of the Location
     * @return where
     */
    public Coordinates getCoordinates(){
        return where;
    }

    /**
     * Builder of Location objects
     */
    public static class LocationBuilder{
        private JSONObject datas;
        private String type;

        /**
         * Constructor of the class LocationBuilder
         * Stock Datas from a JSON object given
         * @param datas contains datas to construct a Locatiop object
         */
        public LocationBuilder(JSONObject datas, String type){
            this.datas = datas;
            this.type = type;
        }

        /**
         * Build a Location object with the datas obtained
         * @return a new Location object
         */
        public Location build(){
            int id = datas.getInt("id");
            Type type = Type.valueOf(this.type);
            String name = datas.getString("name");
            double score = datas.getDouble("score");

            JSONObject coordinatesJSON = new JSONObject(datas.get("coordinates"));
            Coordinates.CoordinatesBuilder coordinates = new Coordinates.CoordinatesBuilder(coordinatesJSON);

            double distance = datas.getDouble("distance");

            return new Location(id, type, name, score, coordinates.build(), distance);
        }
    }
}
