package ch.hepia.api.transport;

import org.json.JSONObject;

import java.io.Serializable;
import java.sql.Time;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

public final class Prognosis implements Serializable {
    private int plateform;
    private Time departure;
    private Time arrival;
    private int capacity1st;
    private int capacity2nd;

    /**
     * Constructor of the Prognosis class
     * A Prognosis contains information on the status of a Stop Object
     * @param plateform plateform where the transport should arrive and leave
     * @param departure time of departure
     * @param arrival time of arrival
     * @param capacity1st number of seats in 1st class
     * @param capacity2nd number of seats in 2nd class
     */
    private Prognosis(int plateform, Time departure, Time arrival, int capacity1st, int capacity2nd){
        this.plateform = plateform;
        this.departure = departure;
        this.arrival = arrival;
        this.capacity1st = capacity1st;
        this.capacity2nd = capacity2nd;
    }

    /**
     * Builder of Prognosis Objects
     */
    public static class PrognosisBuilder{
        private JSONObject datas;

        /**
         * Constructor of the class PrognosisBuilder
         * Stock Datas from a JSON object given
         * @param datas contains datas to construct a Prognosis object
         */
        public PrognosisBuilder(JSONObject datas){
            this.datas = datas;
        }

        /**
         * Build a Prognosis object from the datas obtained
         * @return a new Prognosis object
         * @throws ParseException
         */
        public Prognosis build() throws ParseException {
            int plateform = datas.getInt("plateform");

            DateFormat formatter = new SimpleDateFormat("dd:kk:mm:ss");
            String depatureString = datas.getString("departure").replace('d',':');
            Time departure = new Time(formatter.parse(depatureString).getTime());

            String arrivalString = datas.getString("arrival").replace('d', ':');
            Time arrival = new Time(formatter.parse(arrivalString).getTime());

            int capacity1st = datas.getInt("capacity1st");
            int capacity2nd = datas.getInt("capacity2nd");

            return new Prognosis(plateform, departure, arrival, capacity1st, capacity2nd);
        }
    }
}
