package ch.hepia.events;

import ch.hepia.models.User;
import ch.hepia.api.transport.Section;

import java.util.ArrayList;
import java.util.List;
import java.io.Serializable;

/**
 * Represents an event where a user has left a journey.
 * A journey is composed of several sections
 * (Genève-Vauderens : Genève-Morges is the first section, Morges-Vauderens is the second.)
 */
public class LeftJourney implements Serializable {
    private static final long serialVersionUID = 0xAEF34565674L;
    private User user;
    private List<Section> sections;

    /**
     * Main constructor
     * @param user The user triggering the event
     * @param sections The sections of the journey
     */
    public LeftJourney(User user, List<Section> sections){
        this.user = user;
        this.sections = new ArrayList<>(sections);
    }

	/**
	 * Returns the user
	 * @return the user
	 */
	public User getUser(){
		return user;
	}

	/**
	 * Returns the list of sections of this journey.
	 * @return the list of sections of this journey
	 */
	public List<Section> getSections(){
		return new ArrayList<>(sections);
	}

    @Override
    public String toString() {
        // TODO
        return "welp";
    }
}
