package ch.hepia.mq;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;
import java.io.ByteArrayOutputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.ByteArrayInputStream;
import java.io.Serializable;

import com.rabbitmq.client.ConnectionFactory;
import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.DeliverCallback;

public abstract class MessageQueue {
    private final String exchange;
    private final Connection mqConnection;
    private final Channel mqChannel;
    private final List<Consumer<byte[]>> consumers;

    public MessageQueue(String host, String username, String password, String exchange) throws Exception {
        this.consumers = new ArrayList<>();
        this.exchange = exchange;
        ConnectionFactory queueConnector = new ConnectionFactory();
        queueConnector.setHost(host);
        queueConnector.setUsername(username);
        queueConnector.setPassword(password);

        this.mqConnection = queueConnector.newConnection();
        this.mqChannel = this.mqConnection.createChannel();
        String queueName = this.mqChannel.queueDeclare().getQueue();
        this.mqChannel.queueBind(queueName, this.exchange, "");
        DeliverCallback deliverCallback = (consumerTag, delivery) -> {
            for (Consumer<byte[]> consumer : this.consumers) {
                consumer.accept(delivery.getBody());
            }
        };
        this.mqChannel.basicConsume(queueName, true, deliverCallback, consumerTag -> {
        });
    }

    protected void addConsumer(Consumer<byte[]> messageHandler) {
        this.consumers.add(messageHandler);
    }

    protected void sendBytes(byte[] bytes) throws Exception {
        this.mqChannel.basicPublish(this.exchange, "", null, bytes);
    }

    public void close() throws Exception {
        this.mqChannel.close();
        this.mqConnection.close();
    }

    public static <T extends Serializable> byte[] serialize(T object){
        try (
            ByteArrayOutputStream byteStream = new ByteArrayOutputStream();    
            ObjectOutputStream oos = new ObjectOutputStream(byteStream)
        ) {
            oos.writeObject(object);
            oos.flush();
            return byteStream.toByteArray();
        } catch (final Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
        return null;
    }

    public static <T extends Serializable> T unserialize(byte[] serializedData){
        T result = null;
        try (
            ByteArrayInputStream inputStream = new ByteArrayInputStream(serializedData);
            ObjectInputStream ois = new ObjectInputStream(inputStream)
        ) {            
            result = (T) ois.readObject();
        } catch (final Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
        return result;
    }
}